@@ -3,14 +3,17 @@ package DBIx::Class::Schema::Loader;
 use strict;
 use warnings;
 use base qw/DBIx::Class::Schema Class::Accessor::Grouped/;
-use Carp::Clan qw/^DBIx::Class/;
+use MRO::Compat;
 use mro 'c3';
-use Scalar::Util qw/ weaken /;
+use Carp::Clan qw/^DBIx::Class/;
+use Scalar::Util 'weaken';
+use Sub::Name 'subname';
+use namespace::clean;
 
 # Always remember to do all digits for the version even if they're 0
 # i.e. first release of 0.XX *must* be 0.XX000. This avoids fBSD ports
 # brain damage and presumably various other packaging systems too
-our $VERSION = '0.07010';
+our $VERSION = '0.07011';
 
 __PACKAGE__->mk_group_accessors('inherited', qw/
                                 _loader_args
@@ -25,7 +28,7 @@ __PACKAGE__->_loader_args({});
 
 =head1 NAME
 
-DBIx::Class::Schema::Loader - Dynamic definition of a DBIx::Class::Schema
+DBIx::Class::Schema::Loader - Create a DBIx::Class::Schema based on a database
 
 =head1 SYNOPSIS
 
@@ -46,6 +49,7 @@ DBIx::Class::Schema::Loader - Dynamic definition of a DBIx::Class::Schema
   # from the command line or a shell script with dbicdump (distributed
   # with this module).  Do `perldoc dbicdump` for usage.
   dbicdump -o dump_directory=./lib \
+           -o components='["InflateColumn::DateTime"]' \
            -o debug=1 \
            My::Schema \
            'dbi:Pg:dbname=foo' \
@@ -75,15 +79,17 @@ DBIx::Class::Schema::Loader - Dynamic definition of a DBIx::Class::Schema
 =head1 DESCRIPTION 
 
 DBIx::Class::Schema::Loader automates the definition of a
-L<DBIx::Class::Schema> by scanning database table definitions and
-setting up the columns, primary keys, and relationships.
+L<DBIx::Class::Schema> by scanning database table definitions and setting up
+the columns, primary keys, unique constraints and relationships.
 
 See L<dbicdump> for the C<dbicdump> utility.
 
-DBIx::Class::Schema::Loader currently supports only the DBI storage type.  It
+DBIx::Class::Schema::Loader currently supports only the DBI storage type. It
 has explicit support for L<DBD::Pg>, L<DBD::mysql>, L<DBD::DB2>,
+L<DBD::Firebird>, L<DBD::InterBase>, L<DBD::Informix>, L<DBD::SQLAnywhere>,
 L<DBD::SQLite>, L<DBD::Sybase> (for Sybase ASE and MSSSQL), L<DBD::ODBC> (for
-MSSQL) and L<DBD::Oracle>.  Other DBI drivers may function to a greater or
+MSSQL, MSAccess, Firebird and SQL Anywhere) L<DBD::ADO> (for MSSQL and
+MSAccess) and L<DBD::Oracle>.  Other DBI drivers may function to a greater or
 lesser degree with this loader, depending on how much of the DBI spec they
 implement, and how standard their implementation is.
 
@@ -92,20 +98,25 @@ Patches to make other DBDs work correctly welcome.
 See L<DBIx::Class::Schema::Loader::DBI::Writing> for notes on writing
 your own vendor-specific subclass for an unsupported DBD driver.
 
-This module requires L<DBIx::Class> 0.07006 or later, and obsoletes
-the older L<DBIx::Class::Loader>.
+This module requires L<DBIx::Class> 0.08127 or later, and obsoletes the older
+L<DBIx::Class::Loader>.
 
-This module is designed more to get you up and running quickly against
-an existing database, or to be effective for simple situations, rather
-than to be what you use in the long term for a complex database/project.
-
-That being said, transitioning your code from a Schema generated by this
-module to one that doesn't use this module should be straightforward and
-painless, so don't shy away from it just for fears of the transition down
-the road.
+See L<DBIx::Class::Schema::Loader::Base> for available options.
 
 =head1 METHODS
 
+=head2 loader
+
+The loader object, as class data on your Schema. For methods available see
+L<DBIx::Class::Schema::Loader::Base> and L<DBIx::Class::Schema::Loader::DBI>.
+
+=cut
+
+sub loader {
+    my $self = shift;
+    $self->_loader(@_);
+}
+
 =head2 loader_class
 
 =over 4
@@ -164,7 +175,7 @@ sub _invoke_loader {
     weaken($args->{schema}) if ref $self;
     $args->{dump_directory} ||= $self->dump_to_dir;
     $args->{naming} = $self->naming if $self->naming;
-    $args->{use_namespaces} = $self->use_namespaces if $self->use_namespaces;
+    $args->{use_namespaces} = $self->use_namespaces if defined $self->use_namespaces;
 
     # XXX this only works for relative storage_type, like ::DBI ...
     my $loader_class = $self->loader_class;
@@ -177,8 +188,8 @@ sub _invoke_loader {
     eval { $self->ensure_class_loaded($impl) };
     croak qq/Could not load loader_class "$impl": "$@"/ if $@;
 
-    $self->_loader($impl->new(%$args));
-    $self->_loader->load;
+    $self->loader($impl->new(%$args));
+    $self->loader->load;
     $self->_loader_invoked(1);
 
     $self;
@@ -207,10 +218,11 @@ as soon as the connection information is defined.
 =cut
 
 sub connection {
-    my $self = shift;
+    my $self  = shift;
+    my $class = ref $self || $self;
 
     if($_[-1] && ref $_[-1] eq 'HASH') {
-        for my $option (qw/ loader_class loader_options result_base_class schema_base_class/) {
+        for my $option (qw/loader_class loader_options/) {
             if(my $value = delete $_[-1]->{$option}) {
                 $self->$option($value);
             }
@@ -218,9 +230,50 @@ sub connection {
         pop @_ if !keys %{$_[-1]};
     }
 
-    $self = $self->next::method(@_);
+    # Make sure we inherit from schema_base_class and load schema_components
+    # before connecting.
+    require DBIx::Class::Schema::Loader::Base;
+    my $temp_loader = DBIx::Class::Schema::Loader::Base->new(
+        %{ $self->_loader_args },
+        schema => $self,
+        naming => 'current',
+        use_namespaces => 1,
+    );
+
+    if ($temp_loader->schema_base_class || $temp_loader->schema_components) {
+        my @components = @{ $temp_loader->schema_components }
+            if $temp_loader->schema_components;
+
+        push @components, ('+'.$temp_loader->schema_base_class)
+            if $temp_loader->schema_base_class;
+
+        $class->load_components(@components);
+
+        # This hack is necessary because we changed @ISA of $self through
+        # ->load_components and we are now in a different place in the mro.
+        no warnings 'redefine';
+
+        local *connection = subname __PACKAGE__.'::connection' => sub {
+            my $self = shift;
+            $self->next::method(@_);
+        };
+
+        my @linear_isa = @{ mro::get_linear_isa($class) };
+
+        my $next_method;
+
+        foreach my $i (1..$#linear_isa) {
+            no strict 'refs';
+            $next_method = *{$linear_isa[$i].'::connection'}{CODE};
+            last if $next_method;
+        }
+
+        $self = $self->$next_method(@_);
+    }
+    else {
+        $self = $self->next::method(@_);
+    }
 
-    my $class = ref $self || $self;
     if(!$class->_loader_invoked) {
         $self->_invoke_loader
     }
@@ -409,7 +462,7 @@ Returns a list of the new monikers added.
 
 =cut
 
-sub rescan { my $self = shift; $self->_loader->rescan($self) }
+sub rescan { my $self = shift; $self->loader->rescan($self) }
 
 =head2 naming
 
@@ -453,19 +506,7 @@ Can be imported into your dump script and called as a function as well:
 
 =head2 Multiple Database Schemas
 
-Currently the loader is limited to working within a single schema
-(using the underlying RDBMS's definition of "schema").  If you have a
-multi-schema database with inter-schema relationships (which is easy
-to do in PostgreSQL or DB2 for instance), you currently can only
-automatically load the tables of one schema, and relationships to
-tables in other schemas will be silently ignored.
-
-At some point in the future, an intelligent way around this might be
-devised, probably by allowing the C<db_schema> option to be an
-arrayref of schemas to load.
-
-In "normal" L<DBIx::Class::Schema> usage, manually-defined
-source classes and relationships have no problems crossing vendor schemas.
+See L<DBIx::Class::Schema::Loader::Base/db_schema>.
 
 =head1 ACKNOWLEDGEMENTS
 
@@ -488,6 +529,8 @@ arcanez: Justin Hunter <justin.d.hunter@gmail.com>
 
 ash: Ash Berlin <ash@cpan.org>
 
+btilly: Ben Tilly <btilly@gmail.com>
+
 Caelum: Rafael Kitover <rkitover@cpan.org>
 
 TSUNODA Kazuya <drk@drk7.jp>
@@ -506,6 +549,8 @@ timbunce: Tim Bunce <timb@cpan.org>
 
 mst: Matt S. Trout <mst@shadowcatsystems.co.uk>
 
+mstratman: Mark A. Stratman <stratman@gmail.com>
+
 kane: Jos Boumans <kane@cpan.org>
 
 waawaamilk: Nigel McNie <nigel@mcnie.name>
@@ -524,7 +569,7 @@ spb: Stephen Bennett <spb@exherbo.org>
 
 Matias E. Fernandez <mfernandez@pisco.ch>
 
-mstratman: Mark A. Stratman <stratman@gmail.com>
+alnewkirk: Al Newkirk <awncorp@cpan.org>
 
 ... and lots of other folks. If we forgot you, please write the current
 maintainer or RT.
