@@ -4,10 +4,8 @@ use strict;
 use warnings;
 use base 'DBIx::Class::Schema::Loader::DBI';
 use mro 'c3';
-use Carp::Clan qw/^DBIx::Class/;
-use namespace::clean;
 
-our $VERSION = '0.07010';
+our $VERSION = '0.07011';
 
 =head1 NAME
 
