@@ -3,12 +3,12 @@ package DBIx::Class::Schema::Loader::DBI::ODBC::Microsoft_SQL_Server;
 use strict;
 use warnings;
 use base qw/
+    DBIx::Class::Schema::Loader::DBI::ODBC
     DBIx::Class::Schema::Loader::DBI::MSSQL
 /;
-use Carp::Clan qw/^DBIx::Class/;
 use mro 'c3';
 
-our $VERSION = '0.07010';
+our $VERSION = '0.07011';
 
 =head1 NAME
 
