@@ -3,12 +3,12 @@ package DBIx::Class::Schema::Loader::DBI::ODBC::SQL_Anywhere;
 use strict;
 use warnings;
 use base qw/
+    DBIx::Class::Schema::Loader::DBI::ODBC
     DBIx::Class::Schema::Loader::DBI::SQLAnywhere
 /;
-use Carp::Clan qw/^DBIx::Class/;
 use mro 'c3';
 
-our $VERSION = '0.07010';
+our $VERSION = '0.07011';
 
 =head1 NAME
 
@@ -21,6 +21,23 @@ Proxy for L<DBIx::Class::Schema::Loader::DBI::SQLAnywhere> when using L<DBD::ODB
 
 See L<DBIx::Class::Schema::Loader::Base> for usage information.
 
+=cut
+
+sub _columns_info_for {
+    my $self = shift;
+
+    my $result = $self->next::method(@_);
+
+    while (my ($col, $info) = each %$result) {
+        # The ODBC driver sets the default value to NULL even when it was not specified.
+        if (ref $info->{default_value} && ${ $info->{default_value} } eq 'null') {
+            delete $info->{default_value};
+        }
+    }
+
+    return $result;
+}
+
 =head1 SEE ALSO
 
 L<DBIx::Class::Schema::Loader::DBI::SQLAnywhere>,
@@ -39,3 +56,4 @@ the same terms as Perl itself.
 =cut
 
 1;
+# vim:et sw=4 sts=4 tw=0:
