@@ -6,28 +6,18 @@ use base qw/
     DBIx::Class::Schema::Loader::DBI::Component::QuotedDefault
     DBIx::Class::Schema::Loader::DBI
 /;
-use Carp::Clan qw/^DBIx::Class/;
 use mro 'c3';
 
-our $VERSION = '0.07010';
+our $VERSION = '0.07011';
 
 =head1 NAME
 
 DBIx::Class::Schema::Loader::DBI::Pg - DBIx::Class::Schema::Loader::DBI
 PostgreSQL Implementation.
 
-=head1 SYNOPSIS
-
-  package My::Schema;
-  use base qw/DBIx::Class::Schema::Loader/;
-
-  __PACKAGE__->loader_options( debug => 1 );
-
-  1;
-
 =head1 DESCRIPTION
 
-See L<DBIx::Class::Schema::Loader::Base>.
+See L<DBIx::Class::Schema::Loader> and L<DBIx::Class::Schema::Loader::Base>.
 
 =cut
 
@@ -36,7 +26,7 @@ sub _setup {
 
     $self->next::method(@_);
 
-    $self->{db_schema} ||= 'public';
+    $self->{db_schema} ||= ['public'];
 
     if (not defined $self->preserve_case) {
         $self->preserve_case(0);
@@ -47,6 +37,12 @@ sub _setup {
     }
 }
 
+sub _system_schemas {
+    my $self = shift;
+
+    return ($self->next::method(@_), 'pg_catalog');
+}
+
 sub _table_uniq_info {
     my ($self, $table) = @_;
 
@@ -55,18 +51,17 @@ sub _table_uniq_info {
         if $DBD::Pg::VERSION >= 1.50;
 
     my @uniqs;
-    my $dbh = $self->schema->storage->dbh;
 
     # Most of the SQL here is mostly based on
     #   Rose::DB::Object::Metadata::Auto::Pg, after some prodding from
     #   John Siracusa to use his superior SQL code :)
 
-    my $attr_sth = $self->{_cache}->{pg_attr_sth} ||= $dbh->prepare(
+    my $attr_sth = $self->{_cache}->{pg_attr_sth} ||= $self->dbh->prepare(
         q{SELECT attname FROM pg_catalog.pg_attribute
         WHERE attrelid = ? AND attnum = ?}
     );
 
-    my $uniq_sth = $self->{_cache}->{pg_uniq_sth} ||= $dbh->prepare(
+    my $uniq_sth = $self->{_cache}->{pg_uniq_sth} ||= $self->dbh->prepare(
         q{SELECT x.indrelid, i.relname, x.indkey
         FROM
           pg_catalog.pg_index x
@@ -83,7 +78,7 @@ sub _table_uniq_info {
           c.relname     = ?}
     );
 
-    $uniq_sth->execute($self->db_schema, $table);
+    $uniq_sth->execute($table->schema, $table);
     while(my $row = $uniq_sth->fetchrow_arrayref) {
         my ($tableid, $indexname, $col_nums) = @$row;
         $col_nums =~ s/^\s+//;
@@ -108,29 +103,38 @@ sub _table_uniq_info {
 }
 
 sub _table_comment {
-    my ( $self, $table ) = @_;
-     my ($table_comment) = $self->schema->storage->dbh->selectrow_array(
-        q{SELECT obj_description(oid) 
-            FROM pg_class 
-            WHERE relname=? AND relnamespace=(
-                SELECT oid FROM pg_namespace WHERE nspname=?)
-        }, undef, $table, $self->db_schema
-        );   
+    my $self = shift;
+    my ($table) = @_;
+
+    my $table_comment = $self->next::method(@_);
+
+    return $table_comment if $table_comment;
+
+    ($table_comment) = $self->dbh->selectrow_array(<<'EOF', {}, $table->name, $table->schema);
+SELECT obj_description(oid) 
+FROM pg_class 
+WHERE relname=? AND relnamespace=(SELECT oid FROM pg_namespace WHERE nspname=?)
+EOF
+
     return $table_comment
 }
 
 
 sub _column_comment {
-    my ( $self, $table, $column_number ) = @_;
-     my ($table_oid) = $self->schema->storage->dbh->selectrow_array(
-        q{SELECT oid
-            FROM pg_class 
-            WHERE relname=? AND relnamespace=(
-                SELECT oid FROM pg_namespace WHERE nspname=?)
-        }, undef, $table, $self->db_schema
-        );   
-    return $self->schema->storage->dbh->selectrow_array('SELECT col_description(?,?)', undef, $table_oid,
-    $column_number );
+    my $self = shift;
+    my ($table, $column_number, $column_name) = @_;
+
+    my $column_comment = $self->next::method(@_);
+
+    return $column_comment if $column_comment;
+
+    my ($table_oid) = $self->dbh->selectrow_array(<<'EOF', {}, $table->name, $table->schema);
+SELECT oid
+FROM pg_class 
+WHERE relname=? AND relnamespace=(SELECT oid FROM pg_namespace WHERE nspname=?)
+EOF
+
+    return $self->dbh->selectrow_array('SELECT col_description(?,?)', {}, $table_oid, $column_number);
 }
 
 # Make sure data_type's that don't need it don't have a 'size' column_info, and
@@ -147,7 +151,7 @@ sub _columns_info_for {
         # these types are fixed size
         # XXX should this be a negative match?
         if ($data_type =~
-/^(?:bigint|int8|bigserial|serial8|boolean|bool|box|bytea|cidr|circle|date|double precision|float8|inet|integer|int|int4|line|lseg|macaddr|money|path|point|polygon|real|float4|smallint|int2|serial|serial4|text)\z/i) {
+/^(?:bigint|int8|bigserial|serial8|bool(?:ean)?|box|bytea|cidr|circle|date|double precision|float8|inet|integer|int|int4|line|lseg|macaddr|money|path|point|polygon|real|float4|smallint|int2|serial|serial4|text)\z/i) {
             delete $info->{size};
         }
 # for datetime types, check if it has a precision or not
@@ -171,7 +175,7 @@ EOF
                     delete $info->{size};
                 }
                 else {
-                    my ($integer_datetimes) = $self->schema->storage->dbh
+                    my ($integer_datetimes) = $self->dbh
                         ->selectrow_array('show integer_datetimes');
 
                     my $max_precision =
@@ -195,8 +199,7 @@ EOF
         elsif ($data_type =~ /^(?:bit(?: varying)?|varbit)\z/i) {
             $info->{data_type} = 'varbit' if $data_type =~ /var/i;
 
-            my ($precision) = $self->schema->storage->dbh
-                ->selectrow_array(<<EOF, {}, $table, $col);
+            my ($precision) = $self->dbh->selectrow_array(<<EOF, {}, $table, $col);
 SELECT character_maximum_length
 FROM information_schema.columns
 WHERE table_name = ? and column_name = ?
@@ -231,10 +234,10 @@ SELECT typtype
 FROM pg_catalog.pg_type
 WHERE typname = ?
 EOF
-            if ($typetype eq 'e') {
+            if ($typetype && $typetype eq 'e') {
                 # The following will extract a list of allowed values for the
                 # enum.
-                my $typevalues = $self->schema->storage->dbh
+                my $typevalues = $self->dbh
                     ->selectall_arrayref(<<EOF, {}, $info->{data_type});
 SELECT e.enumlabel
 FROM pg_catalog.pg_enum e
@@ -254,7 +257,8 @@ EOF
         }
 
 # process SERIAL columns
-        if (ref($info->{default_value}) eq 'SCALAR' && ${ $info->{default_value} } =~ /\bnextval\(['"]([.\w]+)/i) {
+        if (ref($info->{default_value}) eq 'SCALAR'
+                && ${ $info->{default_value} } =~ /\bnextval\('([^:]+)'/i) {
             $info->{is_auto_increment} = 1;
             $info->{sequence}          = $1;
             delete $info->{default_value};
@@ -268,6 +272,18 @@ EOF
             my $now = 'now()';
             $info->{original}{default_value} = \$now;
         }
+
+# detect 0/1 for booleans and rewrite
+        if ($data_type =~ /^bool/i && exists $info->{default_value}) {
+            if ($info->{default_value} eq '0') {
+                my $false = 'false';
+                $info->{default_value} = \$false;
+            }
+            elsif ($info->{default_value} eq '1') {
+                my $true = 'true';
+                $info->{default_value} = \$true;
+            }
+        }
     }
 
     return $result;
